#include "TreeNode.h"
#include "Newick.h"
#include <lapack++.h>
#include <lapack.h>
#include <lafnames.h>
#include <iostream>
#include "Felsenstein.h"
#include "Mjoin.h"
#include "Seq.h"
#include "Timer.h"
#include "NJTree.h"
#include "JC-Tree.h"
#include "cmpressed.h"


using namespace std;


int main(int argc, char **argv){
  /* Precision for printing.... */
  cout.precision(7);
  cout.setf(ios::scientific, ios::floatfield);

  /* Declearing variables.... */
  int numSt = 4, numNds = 0, numLvs = 0, ** PhylMat, **SeqMat, nIter = 50, nZell = 30;
  int SeqLen = 0, newLn = 0;
  LaVectorDouble BrLenV2;
  long double logL = 0.0, totalLog = 0.0;
  LaSymmMatDouble RateMat(numSt, numSt), Q(numSt, numSt);
  char filename[127], Newick[127], freq[127], TsTv[127];
  Alinements * InputData;
  LaVectorDouble FreqVec(numSt), Tht(Choose(numSt, 2)), 
    ThtUpdt(Choose(numSt, 2));
  int **PhlMat, **cmpMatrix;

  /* Reading input files and arguements.... */
  strcpy(freq, "yes");
  strcpy(TsTv, "yes");
  if(argc < 2){
    cerr << "Please enter an input file name!" << endl;
    exit(0);
  }
  strcpy(filename, argv[1]);
  strcpy(Newick, "no");
  
  
  /* Setting the GTR model..... */
  FreqVec(0) = 0.2; FreqVec(1) = 0.3;  FreqVec(2) = 0.4;  FreqVec(3) = 0.1;

  Tht(0) = 8;  Tht(1) = 1; Tht(2) = 2;  Tht(3) = 1;  Tht(4) = 1; Tht(5) = 8;
//   RateMat = GTRRateMFct(FreqVec, Tht, numSt);
//   cout << fixed << RateMat << endl;
  
  /* Initializing variables...... */  
  PhylMat = new int*[numLvs + numNds];
  for(int i = 0; i < (numLvs + numNds); i++)
    PhylMat[i] = new int[4];
  for(int i = 0; i < (numLvs + numNds); i++)
    for(int j = 0; j < 4; j++) 
      PhylMat[i][j] = 0;
  
  numLvs = 0;
  
  /* Writing tree matrix from DNA sequences using MJOIN to get initial Branchlengths..... */
  PhylMat = writeTreeMatrix(filename, Newick, numLvs, BrLenV2, &InputData,
			    FreqVec(2), FreqVec(3),FreqVec(0), FreqVec(1),  
			    freq, TsTv);
  numNds = numLvs - 1; 
  int dim = (2 * numLvs) - 1; 
  //  cout << dim << endl;
//     for(int i = 0; i < dim; i++){
//       for(int j = 0; j < 4; j++)
//         cout << PhylMat[i][j] << " ";
//       cout << endl;
//   }
  
  /* Define variables .... */
  LaGenMatDouble CndProbMat(numNds + numLvs - 1, numSt * numSt),
    totalCndProbMat(numNds + numLvs - 1, numSt * numSt), 
    SuffStatMat((numNds + numLvs - 1), numSt * numSt), 
    FMat((numNds + numLvs - 1), Choose(numSt, 2)), 
    GMat((numNds + numLvs - 1), Choose(numSt, 2));
  LaGenMatDouble InvFEgnVc(numSt, numSt), TspFEgnVc(numSt, numSt), 
    EgnVc(numSt, numSt);
  LaVectorDouble EgnVl(numSt);
  LaGenMatDouble  hold(1, numSt * numSt), hold2(1, numSt * numSt);
  int tmpVec[numLvs], nObs = 0;
  double Fsum = 0.0, Gsum = 0.0;

  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      totalCndProbMat(i, j) = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      SuffStatMat(i, j) = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      FMat(i, j) = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      GMat(i, j) = 0.0;
 
  SeqMat = DNAMat(InputData, numLvs, SeqLen);
  cmpMatrix = cmpressed(SeqMat, numLvs, newLn, SeqLen);
//   for(int i = 0; i < (numLvs + 1); i++){
//     for(int j = 0; j < newLn; j++)
//       cout << cmpMatrix[i][j] << " ";
//     cout << endl;
//   }

  /* Get emp freq.... */
  getEmpFreq(FreqVec, SeqMat,  SeqLen, numLvs, numSt);
  cout << "Estimated Frq: "; 
  cout << fixed << FreqVec(0) << " " << FreqVec(1) << " " << FreqVec(2) << " " << FreqVec(3) << endl;

  /* Big iteration loop... */

  for(int iter = 0; iter < nIter; iter++){
    RateMat = GTRRateMFct(FreqVec, Tht, numSt);
    //     cout << fixed << RateMat << endl;
    computeEvalEvec(RateMat, InvFEgnVc, TspFEgnVc, EgnVl, EgnVc, FreqVec, numSt);
    
    //     cout << fixed << InvFEgnVc << endl;
    //     cout << endl << fixed << TspFEgnVc << endl;
    //     cout << endl << fixed << EgnVc << endl;
    //     cout << endl << fixed << EgnVl << endl;
    
    for(int site = 0; site < newLn; site++){
      for(int j = 0; j < numLvs; j++) tmpVec[j] = 0;
      
      for(int j = 0; j < numLvs; j++) tmpVec[j] = cmpMatrix[j][site];
      //       for(int j = 0; j < numLvs; j++) cout << tmpVec[j] << " ";
      //       cout << endl; exit(0);
//       FlsnFct(PhylMat, RateMat, FreqVec, BrLenV2, tmpVec, numSt, numLvs,
// 	      numNds, CndProbMat, logL);
      
      FstFlsnFct(PhylMat, EgnVl, InvFEgnVc, TspFEgnVc, BrLenV2, FreqVec, 
		 tmpVec, numSt, numLvs, numNds, CndProbMat, logL);
      
      nObs = cmpMatrix[numLvs][site];
      
      for(int i = 0; i < (numLvs + numNds - 1); i++)
	for(int j = 0; j < (numSt * numSt); j++)
	  totalCndProbMat(i, j) += ((double) nObs) * CndProbMat(i, j);
      totalLog += ((double) nObs) * logL;
    }
    cout << "logL: " << totalLog << endl;
    //    cout << fixed << CndProbMat << endl; exit(0);

    for(int j = 0; j < (numLvs + numNds-1); j++){
      for(int jj = 0; jj < (numSt * numSt); jj++)
	{hold(0, jj) = 0.0; hold2(0, jj) = 0.0; }

      for(int i = 0; i < numSt; i++)
	for(int k = 0; k < numSt; k++) Q(i, k) = RateMat(i, k) * BrLenV2(j);

      for(int jj = 0; jj < (numSt * numSt); jj++)
	hold(0, jj) = CndProbMat(j, jj);
      
      cout << fixed << Q << endl<< BrLenV2(j) <<endl; 
//     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
//     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl;
    hold2 = hold * Transposition(CndStatFct(FreqVec, Q, numSt), numSt * numSt);     // cout << fixed << hold2 << endl<< endl;
      for(int jj = 0; jj < (numSt * numSt); jj++)
	SuffStatMat(j, jj) = hold2(0, jj);
    }

//     cout << fixed << SuffStatMat << endl<< endl;
//     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
//     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl; exit(0);


    for(int j = 0; j < (numLvs + numNds-1); j++){  
      FMat(j, 0) = SuffStatMat(j, 4) + SuffStatMat(j, 7);
      FMat(j, 1) = SuffStatMat(j, 5) + SuffStatMat(j, 10);
      FMat(j, 2) = SuffStatMat(j, 6) + SuffStatMat(j, 13);
      FMat(j, 3) = SuffStatMat(j, 8) + SuffStatMat(j, 11);
      FMat(j, 4) = SuffStatMat(j, 9) + SuffStatMat(j, 14);
      FMat(j, 5) = SuffStatMat(j, 12) + SuffStatMat(j, 15);
	}
    for(int j = 0; j < (numLvs + numNds - 1); j++){ 
      GMat(j, 0) = (SuffStatMat(j, 0) * FreqVec(1)) + (SuffStatMat(j, 1) * FreqVec(0));
      GMat(j, 1) = (SuffStatMat(j, 0) * FreqVec(2)) + (SuffStatMat(j, 2) * FreqVec(0));
      GMat(j, 2) = (SuffStatMat(j, 0) * FreqVec(3)) + (SuffStatMat(j, 3) * FreqVec(0));
      GMat(j, 3) = (SuffStatMat(j, 1) * FreqVec(2)) + (SuffStatMat(j, 2) * FreqVec(1));
      GMat(j, 4) = (SuffStatMat(j, 1) * FreqVec(3)) + (SuffStatMat(j, 3) * FreqVec(1));
      GMat(j, 5) = (SuffStatMat(j, 2) * FreqVec(3)) + (SuffStatMat(j, 3) * FreqVec(2));
    }

//     cout << fixed << FMat << endl;
//     cout << fixed << GMat << endl;
    for(int j = 0; j < Choose(numSt, 2); j++) ThtUpdt(j) = 0.0;

    for(int zell = 0; zell < nZell; zell++){
      for(int j = 0; j < Choose(numSt, 2); j++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < (numLvs + numNds - 1); k++){
	  Fsum += FMat(k, j);  Gsum += (GMat(k, j) * BrLenV2(k));
	}
	ThtUpdt(j) = Fsum / Gsum;
      }
      
      for(int j = 0; j < (numLvs + numNds - 1); j++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < Choose(numSt, 2); k++){
	  Fsum += FMat(j, k);  Gsum += (GMat(j, k) * Tht(k));
	}
	BrLenV2(j) = Fsum / Gsum;
      }
      for(int k = 0; k < Choose(numSt, 2); k++)
	Tht(k) = ThtUpdt(k);
      Tht(1) = 1.0;
    }

    cout << "BrLen: "; 
    for(int j = 0; j < (numLvs + numNds - 1); j++)
      cout << fixed << BrLenV2(j) << " ";
    cout << endl << "Tht: ";
    for(int k = 0; k < Choose(numSt, 2); k++)
    cout << fixed << Tht(k) << " ";
    cout << endl << endl;

  }/* End of Big iteration loop... */
  return 0;
}


