/*
  Author: Ruriko Yoshida 
  Code  : This is a C++ version of Asger Hobolth's R code.
  Date  : August 10th., 2006
  Update: Sept 16th, 2006
  Note  : This cpp file will be eventually attached to EMGNJ.cpp
          file.  (i.e. this will be some sort of library.)

Algorithm: 
Input: DNA sequences, and m.
Output: a tree and the rate matrix in the GTR model.
Step 1. Fix m
Step 2. Estimate tree using MJOIN(m) and Jukes-Cantor rate matrix.
Step 3. Estimate rate matrix in GTR model using EM-alg and tree from 2.

*/

#include "TreeNode.h"
#include "Newick.h"
#include <lapack++.h>
#include <lapack.h>
#include <lafnames.h>
#include <iostream>
#include "Felsenstein.h"
#include "Mjoin.h"
#include "Seq.h"
#include "Timer.h"
#include "NJTree.h"
#include "JC-Tree.h"
#include "cmpressed.h"


using namespace std;

int** getSubtree(int** originalTreeMatrix, int n, int m, int* lvIndex, 
		 LaVectorDouble & newBrlen, LaVectorDouble Brlen){

  list<int> lvs, parents, newLvs, hold, subLvs;
  long double wt = 0.0;
  list<int>::iterator nums_iter, nums_iter2, nums_iter3;
  int numLv = (2 * m) - 1, numOfSubtree = Choose(n, m), 
    root = (2 * n) - 1, tmp = 0, left = 0, right = 0, counter = 0;
  int ** PhlMat, **TreeMatrix;
 
  newBrlen.resize(numLv - 1, 1); 
  PhlMat = new int*[numLv];
  for(int i = 0; i < numLv; i++)
    PhlMat[i] = new int[4];
  for(int i = 0; i < numLv; i++)
    for(int j =0; j < 4; j++)
      PhlMat[i][j] = 0;

  TreeMatrix = new int*[(2 * n - 1)];
  for(int i = 0; i < (2 * n - 1); i++)
    TreeMatrix[i] = new int[4];
  for(int i = 0; i < (2 * n - 1); i++)
    for(int j =0; j < 4; j++)
      TreeMatrix[i][j] = originalTreeMatrix[i][j];

  for(int i = 0; i < m; i++)
    PhlMat[i][0] = 1;

  for(int i = 0; i < m; i++){
    lvs.push_back(lvIndex[i]); newLvs.push_back(lvIndex[i]);
  }

  for(int i = 0; i < m; i++)
    parents.push_back(TreeMatrix[lvIndex[i] - 1][3]);

  while(!parents.empty()){

    tmp = parents.front();
    parents.pop_front();
    
    nums_iter = find(parents.begin(), parents.end(), TreeMatrix[tmp - 1][3]);
    if(nums_iter == parents.end())
      if(TreeMatrix[tmp - 1][3] != 0)
	parents.push_back(TreeMatrix[tmp - 1][3]);
    
    nums_iter = find(lvs.begin(), lvs.end(), tmp);

    if(nums_iter == lvs.end()){
      lvs.push_back(tmp);
    }
  }

  lvs.sort();

  while(!lvs.empty()){
    tmp = lvs.back();
    lvs.pop_back(); 
    parents.push_back(tmp);
    left =  TreeMatrix[tmp - 1][1]; //cout << left << endl;
    right =  TreeMatrix[tmp - 1][2];//cout << right << endl; 

    nums_iter2 = find(lvs.begin(), lvs.end(), left);
    nums_iter3 = find(lvs.begin(), lvs.end(), right);
    if((nums_iter2 != lvs.end()) && (nums_iter3 != lvs.end()))
      hold.push_back(tmp);
  }

  hold.sort();

  root = hold.back();
  hold.push_front(root);
  parents.sort();

  hold.clear();

  while(!parents.empty()){
    tmp = parents.back();
    parents.pop_back(); 
    if(tmp <= root){
      left =  TreeMatrix[tmp - 1][1]; //cout << left << endl;
      right =  TreeMatrix[tmp - 1][2];//cout << right << endl << endl;
      nums_iter2 = find(parents.begin(), parents.end(), left);
      nums_iter3 = find(parents.begin(), parents.end(), right);
      if(nums_iter2 != parents.end())
	hold.push_front(tmp);
      if(nums_iter3 != parents.end())
	hold.push_front(tmp);
    }
  }

  while(!hold.empty()){
    tmp = hold.back();
    hold.pop_back();
    nums_iter = find(newLvs.begin(), newLvs.end(), tmp);
    if(nums_iter == newLvs.end())
      newLvs.push_front(tmp);
  }

  newLvs.sort();
//   for(int i = 0; i < (2 * n - 2); i++) 
//     cout << Brlen(i) << " ";
//   cout << endl;
  int flag = 0;
//   for(nums_iter = newLvs.begin(); nums_iter != newLvs.end(); nums_iter++){
//     cout << (*nums_iter) << " ";
//   }cout << endl;

  while(!newLvs.empty()){
    flag = 0;
    tmp = newLvs.back();
    newLvs.pop_back();
    if(tmp > n){
      nums_iter = find(newLvs.begin(), newLvs.end(), TreeMatrix[tmp - 1][1]);
      if(nums_iter != newLvs.end())
	flag++;
      else {
	if(tmp != numLv){
	  //cout << Brlen(TreeMatrix[tmp - 1][2] - 1) << " " << Brlen(tmp - 1) << endl;
	  Brlen(TreeMatrix[tmp - 1][2] - 1) += Brlen(tmp - 1);
	  Brlen(tmp - 1) = 0.0;
	  //cout << TreeMatrix[tmp - 1][2] << " " << tmp<< endl;
	  //cout << Brlen(TreeMatrix[tmp - 1][2] - 1) << " " << Brlen(tmp - 1) << endl;
	}
	TreeMatrix[TreeMatrix[tmp - 1][2] - 1][3] = TreeMatrix[tmp - 1][3];
	if(TreeMatrix[TreeMatrix[tmp - 1][3] - 1][1] == tmp)
	  TreeMatrix[TreeMatrix[tmp - 1][3] - 1][1] = TreeMatrix[tmp - 1][2];
	if(TreeMatrix[TreeMatrix[tmp - 1][3] - 1][2] == tmp)
	  TreeMatrix[TreeMatrix[tmp - 1][3] - 1][2] = TreeMatrix[tmp - 1][2];
      }
      nums_iter = find(newLvs.begin(), newLvs.end(), TreeMatrix[tmp - 1][2]);
      if(nums_iter != newLvs.end())
	flag++;
      else{ 	
	if(tmp != numLv){
	  //cout << Brlen(TreeMatrix[tmp - 1][1] - 1) << " " << Brlen(tmp - 1) << endl;
	  Brlen(TreeMatrix[tmp - 1][1] - 1) += Brlen(tmp - 1);
	  Brlen(tmp - 1) = 0.0;
	  //cout << TreeMatrix[tmp - 1][1] << " " << tmp<< endl;
	  //cout << Brlen(TreeMatrix[tmp - 1][1] - 1) << " " << Brlen(tmp - 1) << endl;
	}
	TreeMatrix[TreeMatrix[tmp - 1][1] - 1][3] = TreeMatrix[tmp - 1][3];
	if(TreeMatrix[TreeMatrix[tmp - 1][3] - 1][1] == tmp)
	  TreeMatrix[TreeMatrix[tmp - 1][3] - 1][1] = TreeMatrix[tmp - 1][1];
	if(TreeMatrix[TreeMatrix[tmp - 1][3] - 1][2] == tmp)
	  TreeMatrix[TreeMatrix[tmp - 1][3] - 1][2] = TreeMatrix[tmp - 1][1];
      }
      //cout << flag << endl;
      if(flag == 2) subLvs.push_front(tmp);
    }
    else {subLvs.push_front(tmp);}
  }
  subLvs.sort();

  int *LvIndex;
  LvIndex = new int[2 * n - 1];
  counter = 1;
  for(nums_iter = subLvs.begin(); nums_iter != subLvs.end(); nums_iter++){
    LvIndex[(*nums_iter) - 1] = counter;
    //cout << (*nums_iter) << " ";
    counter++;
  }//cout << endl;

  for(nums_iter = subLvs.begin(); nums_iter != subLvs.end(); nums_iter++){
    PhlMat[LvIndex[(*nums_iter) - 1] - 1][3] = LvIndex[TreeMatrix[(*nums_iter) - 1][3] - 1];
    if((*nums_iter)> n){
      PhlMat[LvIndex[(*nums_iter) - 1] - 1][1] = LvIndex[TreeMatrix[(*nums_iter) - 1][1] - 1];
      PhlMat[LvIndex[(*nums_iter) - 1] - 1][2] = LvIndex[TreeMatrix[(*nums_iter) - 1][2] - 1];
    }
  }
  PhlMat[(2 * m - 2)][3] = 0;

  counter = 0;
  subLvs.pop_back(); 
  for(nums_iter = subLvs.begin(); nums_iter != subLvs.end(); nums_iter++)
  {
    newBrlen(counter) = Brlen((*nums_iter) - 1);
    //   totalBrlen += Brlen((*nums_iter) - 1);
    counter++;
  }
  subLvs.clear();
//   for(int i = 0; i < (2 * n - 2); i++) 
//     cout << Brlen(i) << " ";
//   cout << endl;

//   for(int i = 0; i < (2 * m - 2); i++) 
//     cout << newBrlen(i) << " ";
//   cout << endl;
  delete []  LvIndex;

//   for(int i = 0; i < numLv; i++){
//     for(int j =0; j < 4; j++)
//       cout << PhlMat[i][j] << " ";
//     cout << endl;
//   }
  for(int i = 0; i < (2 * n - 1); i++)
    delete [] TreeMatrix[i];
  delete [] TreeMatrix;

  return PhlMat;
}

long double GTR(int numSt, int numLvs, int numNds,
		Alinements * InputData, int SeqLen, int** PhylMat,
		LaVectorDouble FreqVec, LaVectorDouble& Tht, int nIter,
		LaVectorDouble& BrLenV2, int nZell, double **GMat, double **FMat){

  /* Define variables .... */
  LaGenMatDouble CndProbMat(numNds + numLvs - 1, numSt * numSt),
    totalCndProbMat(numNds + numLvs - 1, numSt * numSt); 
    //SuffStatMat((numNds + numLvs - 1), numSt * numSt);
//   double FMat[(numNds + numLvs - 1)][Choose(numSt, 2)], 
//     GMat[(numNds + numLvs - 1)][Choose(numSt, 2)];
  double **SuffStatMat;
  LaGenMatDouble InvFEgnVc(numSt, numSt), TspFEgnVc(numSt, numSt), 
    EgnVc(numSt, numSt), tmpMat(numSt * numSt, numSt * numSt);
  LaVectorDouble EgnVl(numSt);
  LaGenMatDouble RateMat(numSt, numSt), Q(numSt, numSt);

  long double totalLog = 0.0, logL = 0.0;
  int **SeqMat, newLn = 0;
  int **cmpMatrix;
  double  *ThtUpdt;  
  int *tmpVec, nObs = 0;
  double Fsum = 0.0, Gsum = 0.0;
  tmpVec = new int[numLvs];

//   double FMat[(numNds + numLvs - 1)][Choose(numSt, 2)], 
//     GMat[(numNds + numLvs - 1)][Choose(numSt, 2)];

  ThtUpdt = new double[Choose(numSt, 2)];
  SuffStatMat = new double*[(numNds + numLvs - 1)];
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    SuffStatMat[i] = new double[numSt * numSt];

  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      totalCndProbMat(i, j) = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      SuffStatMat[i][j] = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      FMat[i][j] = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      GMat[i][j] = 0.0;

  SeqMat = DNAMat(InputData, numLvs, SeqLen);
  cmpMatrix = cmpressed(SeqMat, numLvs, newLn, SeqLen);

  //  getEmpFreq(FreqVec, SeqMat,  SeqLen, numLvs, numSt);

  /* Printing out initial information.... */
//   cout << "Estimated Emp Frq: "; 
//   for(int i = 0; i < numSt; i++)
//   cout << fixed << FreqVec(i) << " ";
//   cout << endl;
//   cout << "Initial Theta: ";
//   for(int i = 0; i < Choose(numSt, 2); i++)
//     cout << fixed << Tht(i) << " ";
//   cout << endl;
//   cout << "Initial Branch lengths: ";
//   for(int i = 0; i < (numLvs + numNds - 1); i++)
//     cout << fixed << BrLenV2(i) << " ";
//   cout << endl;
//   cout << endl;

  /* Big iteration begins.... */
  for(int iter = 0; iter < nIter; iter++){
    //cout << "Iteration: " << iter + 1 << endl;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < (numSt * numSt); j++)
	totalCndProbMat(i, j) = 0.0;
    
    totalLog = 0.0;
    
    /* end of Setting subsets..... */
    
    
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < (numSt * numSt); j++)
	SuffStatMat[i][j] = 0.0;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < Choose(numSt, 2); j++)
	FMat[i][j] = 0.0;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < Choose(numSt, 2); j++)
	GMat[i][j] = 0.0;
    
    RateMat = GTRRateMFct2(FreqVec, Tht, numSt);
    
    computeEvalEvec2(RateMat, InvFEgnVc, TspFEgnVc, EgnVl, EgnVc, FreqVec, numSt);
    // cout << RateMat << endl;
    //     cout << InvFEgnVc << endl << endl;
    //     cout << TspFEgnVc << endl << endl;
    
    for(int site = 0; site < newLn; site++){
      for(int j = 0; j < numLvs; j++) tmpVec[j] = 0;
      
      for(int j = 0; j < numLvs; j++) tmpVec[j] = cmpMatrix[j][site];
      //       for(int j = 0; j < numLvs; j++) cout << tmpVec[j] << " ";
      //       cout << endl; exit(0);
      //       FlsnFct(PhylMat, RateMat, FreqVec, BrLenV2, tmpVec, numSt, numLvs,
      // 	      numNds, CndProbMat, logL);
      
      FstFlsnFct(PhylMat, EgnVl, InvFEgnVc, TspFEgnVc, BrLenV2, FreqVec, 
		 tmpVec, numSt, numLvs, numNds, CndProbMat, logL);
      
      nObs = cmpMatrix[numLvs][site];
      
      for(int i = 0; i < (numLvs + numNds - 1); i++)
	for(int j = 0; j < (numSt * numSt); j++)
	  totalCndProbMat(i, j) += ((double) nObs) * CndProbMat(i, j);
      totalLog += ((double) nObs) * logL;
    }
    
    //cout << "logL: " << totalLog << endl;
    //    cout << fixed << CndProbMat << endl; exit(0);
    
    for(int j = 0; j < (numLvs + numNds-1); j++){
      
      //      for(int k = 0; k < (numSt * numSt); k++) hold[k] = 0.0;
      for(int i = 0; i < numSt; i++)
	for(int k = 0; k < numSt; k++) Q(i, k) = RateMat(i, k) * BrLenV2(j);
      
      for(int i = 0; i < (numSt * numSt); i++)
	for(int k = 0; k < (numSt * numSt); k++) tmpMat(i, k) = 0.0;
      tmpMat = Transposition(CndStatFct2(FreqVec, Q, numSt), numSt*numSt);
      
// 	if(j == 3){
// 	  cout << RateMat << endl;
// 	  cout << Q << endl;
// 	  cout << CndStatFct2(FreqVec, Q, numSt) << endl;
// 	  exit(0);
// 	}
      for(int ii = 0; ii < (numSt * numSt); ii++)
	for(int jj = 0; jj < (numSt * numSt); jj++)
	    // 	  SuffStatMat[j][ii] += totalCndProbMat(j, jj) * (Transposition(CndStatFct(FreqVec, Q, numSt), numSt*numSt))(jj, ii);  
	  SuffStatMat[j][ii] += totalCndProbMat(j, jj) * tmpMat(jj, ii);  
	
      //      cout << fixed << Q << endl<< BrLenV2(j) <<endl; 
      //     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
      //     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl;
      
      //       for(int jj = 0; jj < (numSt * numSt); jj++)
      // 	SuffStatMat(j, jj) = hold[jj];
    }
    
    //     cout << fixed << SuffStatMat << endl<< endl;
    //     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
    //     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl; exit(0);
    
    
    for(int j = 0; j < (numLvs + numNds-1); j++){  
      FMat[j][0] = SuffStatMat[j][4] + SuffStatMat[j][7];
      FMat[j][1] = SuffStatMat[j][5] + SuffStatMat[j][10];
      FMat[j][2] = SuffStatMat[j][6] + SuffStatMat[j][13];
      FMat[j][3] = SuffStatMat[j][8] + SuffStatMat[j][11];
      FMat[j][4] = SuffStatMat[j][9] + SuffStatMat[j][14];
      FMat[j][5] = SuffStatMat[j][12] + SuffStatMat[j][15];
    }
    for(int j = 0; j < (numLvs + numNds - 1); j++){ 
      GMat[j][0] = (SuffStatMat[j][0] * FreqVec(1)) + (SuffStatMat[j][1] * FreqVec(0));
      GMat[j][1] = (SuffStatMat[j][0] * FreqVec(2)) + (SuffStatMat[j][2] * FreqVec(0));
      GMat[j][2] = (SuffStatMat[j][0] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(0));
      GMat[j][3] = (SuffStatMat[j][1] * FreqVec(2)) + (SuffStatMat[j][2] * FreqVec(1));
      GMat[j][4] = (SuffStatMat[j][1] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(1));
      GMat[j][5] = (SuffStatMat[j][2] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(2));
    }
    
    //    cout << fixed << FMat << endl;
    //     cout << fixed << GMat << endl;
    for(int j = 0; j < Choose(numSt, 2); j++) ThtUpdt[j] = 0.0;
    
    for(int zell = 0; zell < nZell; zell++){
      for(int j = 0; j < Choose(numSt, 2); j++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < (numLvs + numNds - 1); k++){
	  Fsum += FMat[k][j];  Gsum += (GMat[k][j] * BrLenV2(k));
	}
	ThtUpdt[j] = Fsum / Gsum;
	//cout << Gsum << endl;
      }
      
      for(int jj = 0; jj < (numLvs + numNds - 1); jj++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < Choose(numSt, 2); k++){
	  Fsum += FMat[jj][k];  Gsum += (GMat[jj][k] * Tht(k));
	}
	BrLenV2(jj) = Fsum / Gsum;//cout << Gsum << endl;
	if(BrLenV2(jj) < 0.0000001) BrLenV2(jj) = 0.0000001;
      }
      for(int k = 0; k < Choose(numSt, 2); k++){
	if(ThtUpdt[k] < 0.0000001) Tht(k) = 0.0000001;
	else
	  Tht(k) = ThtUpdt[k];
      }
      Tht(1) = 1.0;
      
      //     for(int k = 0; k < Choose(numSt, 2); k++)
      //       Tht(k) = ThtUpdt[k];
    }
    
//     cout << "BrLen: "; 
//     for(int j = 0; j < (numLvs + numNds - 1); j++)
//       cout << fixed << BrLenV2(j) << " ";
//     cout << endl << "Tht: ";
//     for(int k = 0; k < Choose(numSt, 2); k++)
//       cout << fixed << Tht(k) << " ";
//     cout << endl << endl;
    
  }/* End of Big iteration loop... */
  // exit(0);
//   delete [] ThtUpdt; 
//   for(int i = 0; i < (numLvs + 1); i++)
//     delete [] cmpMatrix[i];
//   delete [] cmpMatrix;
//   for(int i = 0; i < (numLvs + 1); i++)
//     delete [] SeqMat[i];
//   delete [] SeqMat;
//   for(int i = 0; i < (numLvs + numNds - 1); i++)
//     delete [] SuffStatMat[i];
//   delete []  SuffStatMat;
//   delete [] tmpVec;
//   cout << "BrLen: "; 
//   for(int j = 0; j < (numLvs + numNds - 1); j++)
//     cout << fixed << BrLenV2(j) << " ";
//   cout << endl << "Tht: ";
//   for(int k = 0; k < Choose(numSt, 2); k++)
//     cout << fixed << Tht(k) << " ";
//   cout << endl << endl;
  double Brsum = 0.0;
  for(int i = 0; i < numSt; i++) Brsum -= RateMat(i,i)*FreqVec(i);
  for(int j = 0; j < (numLvs + numNds - 1); j++)
    BrLenV2(j) *= Brsum;

  return totalLog;
}

long double GTR2(int numSt, int numLvs, int numNds,
		Alinements * InputData, int SeqLen, int** PhylMat,
		LaVectorDouble FreqVec, LaVectorDouble& Tht, int nIter,
		LaVectorDouble& BrLenV2, int nZell){

  /* Define variables .... */
  LaGenMatDouble CndProbMat(numNds + numLvs - 1, numSt * numSt),
    totalCndProbMat(numNds + numLvs - 1, numSt * numSt); 
    //SuffStatMat((numNds + numLvs - 1), numSt * numSt);
//   double FMat[(numNds + numLvs - 1)][Choose(numSt, 2)], 
//     GMat[(numNds + numLvs - 1)][Choose(numSt, 2)];
  double **SuffStatMat;
  LaGenMatDouble InvFEgnVc(numSt, numSt), TspFEgnVc(numSt, numSt), 
    EgnVc(numSt, numSt), tmpMat(numSt * numSt, numSt * numSt);
  LaVectorDouble EgnVl(numSt);
  LaGenMatDouble RateMat(numSt, numSt), Q(numSt, numSt);

  long double totalLog = 0.0, logL = 0.0;
  int **SeqMat, newLn = 0;
  int **cmpMatrix;
  double  *ThtUpdt;  
  int *tmpVec, nObs = 0;
  double Fsum = 0.0, Gsum = 0.0;
  tmpVec = new int[numLvs];

  double FMat[(numNds + numLvs - 1)][Choose(numSt, 2)], 
    GMat[(numNds + numLvs - 1)][Choose(numSt, 2)];

  ThtUpdt = new double[Choose(numSt, 2)];
  SuffStatMat = new double*[(numNds + numLvs - 1)];
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    SuffStatMat[i] = new double[numSt * numSt];

  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      totalCndProbMat(i, j) = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < (numSt * numSt); j++)
      SuffStatMat[i][j] = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      FMat[i][j] = 0.0;
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    for(int j = 0; j < Choose(numSt, 2); j++)
      GMat[i][j] = 0.0;

  SeqMat = DNAMat(InputData, numLvs, SeqLen);
  cmpMatrix = cmpressed(SeqMat, numLvs, newLn, SeqLen);

  getEmpFreq(FreqVec, SeqMat,  SeqLen, numLvs, numSt);

  /* Printing out initial information.... */
  cout << "Estimated Emp Frq: "; 
  for(int i = 0; i < numSt; i++)
  cout << fixed << FreqVec(i) << " ";
  cout << endl;
  cout << "Initial Theta: ";
  for(int i = 0; i < Choose(numSt, 2); i++)
    cout << fixed << Tht(i) << " ";
  cout << endl;
  cout << "Initial Branch lengths: ";
  for(int i = 0; i < (numLvs + numNds - 1); i++)
    cout << fixed << BrLenV2(i) << " ";
  cout << endl;
  cout << endl;

  /* Big iteration begins.... */
  for(int iter = 0; iter < nIter; iter++){
    //cout << "Iteration: " << iter + 1 << endl;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < (numSt * numSt); j++)
	totalCndProbMat(i, j) = 0.0;
    
    totalLog = 0.0;
    
    /* end of Setting subsets..... */
    
    
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < (numSt * numSt); j++)
	SuffStatMat[i][j] = 0.0;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < Choose(numSt, 2); j++)
	FMat[i][j] = 0.0;
    for(int i = 0; i < (numLvs + numNds - 1); i++)
      for(int j = 0; j < Choose(numSt, 2); j++)
	GMat[i][j] = 0.0;
    
    RateMat = GTRRateMFct2(FreqVec, Tht, numSt);
    
    computeEvalEvec2(RateMat, InvFEgnVc, TspFEgnVc, EgnVl, EgnVc, FreqVec, numSt);
    // cout << RateMat << endl;
    //     cout << InvFEgnVc << endl << endl;
    //     cout << TspFEgnVc << endl << endl;
    
    for(int site = 0; site < newLn; site++){
      for(int j = 0; j < numLvs; j++) tmpVec[j] = 0;
      
      for(int j = 0; j < numLvs; j++) tmpVec[j] = cmpMatrix[j][site];
      //       for(int j = 0; j < numLvs; j++) cout << tmpVec[j] << " ";
      //       cout << endl; exit(0);
      //       FlsnFct(PhylMat, RateMat, FreqVec, BrLenV2, tmpVec, numSt, numLvs,
      // 	      numNds, CndProbMat, logL);
      
      FstFlsnFct(PhylMat, EgnVl, InvFEgnVc, TspFEgnVc, BrLenV2, FreqVec, 
		 tmpVec, numSt, numLvs, numNds, CndProbMat, logL);
      
      nObs = cmpMatrix[numLvs][site];
      
      for(int i = 0; i < (numLvs + numNds - 1); i++)
	for(int j = 0; j < (numSt * numSt); j++)
	  totalCndProbMat(i, j) += ((double) nObs) * CndProbMat(i, j);
      totalLog += ((double) nObs) * logL;
    }
    
    //cout << "logL: " << totalLog << endl;
    //    cout << fixed << CndProbMat << endl; exit(0);
    
    for(int j = 0; j < (numLvs + numNds-1); j++){
      
      //      for(int k = 0; k < (numSt * numSt); k++) hold[k] = 0.0;
      for(int i = 0; i < numSt; i++)
	for(int k = 0; k < numSt; k++) Q(i, k) = RateMat(i, k) * BrLenV2(j);
      
      for(int i = 0; i < (numSt * numSt); i++)
	for(int k = 0; k < (numSt * numSt); k++) tmpMat(i, k) = 0.0;
      tmpMat = Transposition(CndStatFct2(FreqVec, Q, numSt), numSt*numSt);
      
// 	if(j == 3){
// 	  cout << RateMat << endl;
// 	  cout << Q << endl;
// 	  cout << CndStatFct2(FreqVec, Q, numSt) << endl;
// 	  exit(0);
// 	}
      for(int ii = 0; ii < (numSt * numSt); ii++)
	for(int jj = 0; jj < (numSt * numSt); jj++)
	    // 	  SuffStatMat[j][ii] += totalCndProbMat(j, jj) * (Transposition(CndStatFct(FreqVec, Q, numSt), numSt*numSt))(jj, ii);  
	  SuffStatMat[j][ii] += totalCndProbMat(j, jj) * tmpMat(jj, ii);  
	
      //      cout << fixed << Q << endl<< BrLenV2(j) <<endl; 
      //     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
      //     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl;
      
      //       for(int jj = 0; jj < (numSt * numSt); jj++)
      // 	SuffStatMat(j, jj) = hold[jj];
    }
    
    //     cout << fixed << SuffStatMat << endl<< endl;
    //     cout << fixed << CndStatFct(FreqVec, Q, numSt) << endl<< endl; 
    //     cout << fixed << Transposition(CndStatFct(FreqVec, Q, numSt),  numSt * numSt) << endl<< endl; exit(0);
    
    
    for(int j = 0; j < (numLvs + numNds-1); j++){  
      FMat[j][0] = SuffStatMat[j][4] + SuffStatMat[j][7];
      FMat[j][1] = SuffStatMat[j][5] + SuffStatMat[j][10];
      FMat[j][2] = SuffStatMat[j][6] + SuffStatMat[j][13];
      FMat[j][3] = SuffStatMat[j][8] + SuffStatMat[j][11];
      FMat[j][4] = SuffStatMat[j][9] + SuffStatMat[j][14];
      FMat[j][5] = SuffStatMat[j][12] + SuffStatMat[j][15];
    }
    for(int j = 0; j < (numLvs + numNds - 1); j++){ 
      GMat[j][0] = (SuffStatMat[j][0] * FreqVec(1)) + (SuffStatMat[j][1] * FreqVec(0));
      GMat[j][1] = (SuffStatMat[j][0] * FreqVec(2)) + (SuffStatMat[j][2] * FreqVec(0));
      GMat[j][2] = (SuffStatMat[j][0] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(0));
      GMat[j][3] = (SuffStatMat[j][1] * FreqVec(2)) + (SuffStatMat[j][2] * FreqVec(1));
      GMat[j][4] = (SuffStatMat[j][1] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(1));
      GMat[j][5] = (SuffStatMat[j][2] * FreqVec(3)) + (SuffStatMat[j][3] * FreqVec(2));
    }
    
    //    cout << fixed << FMat << endl;
    //     cout << fixed << GMat << endl;
    for(int j = 0; j < Choose(numSt, 2); j++) ThtUpdt[j] = 0.0;
    
    for(int zell = 0; zell < nZell; zell++){
      for(int j = 0; j < Choose(numSt, 2); j++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < (numLvs + numNds - 1); k++){
	  Fsum += FMat[k][j];  Gsum += (GMat[k][j] * BrLenV2(k));
	}
	ThtUpdt[j] = Fsum / Gsum;
	//cout << Gsum << endl;
      }
      
      for(int jj = 0; jj < (numLvs + numNds - 1); jj++){
	Gsum = 0.0; Fsum = 0.0;
	for(int k = 0; k < Choose(numSt, 2); k++){
	  Fsum += FMat[jj][k];  Gsum += (GMat[jj][k] * Tht(k));
	}
	BrLenV2(jj) = Fsum / Gsum;//cout << Gsum << endl;
	if(BrLenV2(jj) < 0.0000001) BrLenV2(jj) = 0.0000001;
      }
      for(int k = 0; k < Choose(numSt, 2); k++){
	if(ThtUpdt[k] < 0.0000001) Tht(k) = 0.0000001;
	else
	  Tht(k) = ThtUpdt[k];
      }
      Tht(1) = 1.0;
      
      //     for(int k = 0; k < Choose(numSt, 2); k++)
      //       Tht(k) = ThtUpdt[k];
    }
    
    cout << "BrLen: "; 
    for(int j = 0; j < (numLvs + numNds - 1); j++)
      cout << fixed << BrLenV2(j) << " ";
    cout << endl << "Tht: ";
    for(int k = 0; k < Choose(numSt, 2); k++)
      cout << fixed << Tht(k) << " ";
    cout << endl << endl;
    
  }/* End of Big iteration loop... */
  // exit(0);
//   delete [] ThtUpdt; 
//   for(int i = 0; i < (numLvs + 1); i++)
//     delete [] cmpMatrix[i];
//   delete [] cmpMatrix;
//   for(int i = 0; i < (numLvs + 1); i++)
//     delete [] SeqMat[i];
//   delete [] SeqMat;
//   for(int i = 0; i < (numLvs + numNds - 1); i++)
//     delete [] SuffStatMat[i];
//   delete []  SuffStatMat;
//   delete [] tmpVec;
  
  return totalLog;
}

int EMGNJ(int argc, char **argv){
  /* Precision for printing.... */
  cout.precision(7);
  cout.setf(ios::scientific, ios::floatfield);

  /* Declearing variables.... */
  int numSt = 4, numNds = 0, m = 3, numLvs = 0, ** PhylMat, **subPhylMat,
    nMWt = 6,
   BignIter = 10, nIter = 10, nZell = 30;
 //  TreeNode* initialTree, *finalTree;
  int SeqLen = 0, newLn = 0, h = 0;
  LaVectorDouble BrLenV2, newBrLen;
  //  long double logL = 0.0, totalLog = 0.0;
  // LaGenMatDouble RateMat(numSt, numSt), Q(numSt, numSt);
  char filename[127], Newick[127], freq[127], TsTv[127], 
    AlignFile[200];
  Alinements * InputData, *subInputData;
  LaVectorDouble FreqVec(numSt), Tht(Choose(numSt, 2)), 
    totalTht(Choose(numSt, 2)); 
  double  trans = 0.5, **GMat, **FMat;
  //int **PhlMat, **cmpMatrix;
  //  ofstream finalTreeFile;
  char ** listOfNames; 
  double tfreq = 0.0, cfreq = 0.0, afreq = 0.0, gfreq = 0.0;
  char invocation[200], mOption[127], mfile[127], iter[127];

  long int numOfSubsets;
  int **subsets;
  long double * totalBrLen;

  Timer ct;

  /* Reading input files and arguements.... */
  strcpy(mOption, "no");
  strcpy(mfile, "no");
  strcpy(iter, "no");
  strcpy(freq, "no");
  strcpy(TsTv, "no");
  strcpy(Newick, "no");

  cout << "EMGNJ v0.9" << endl;
  m = 2;

//   strcpy(filename, argv[argc-1]);

  strcpy(invocation, "invocation: ");
  for(int f = 0; f < argc; f++){
    strcat(invocation, argv[f]);
    strcat(invocation, " ");
    if(strncmp(argv[f], "-m", 2) == 0) 
      { strcpy(mOption, "yes"); 
      if((argv[f + 1] == NULL) || (atoi(argv[f + 1]) < 2) || (atof(argv[f + 1]) != atoi(argv[f+1]))){
	cerr << "Error: Please make sure that m is an integer > 1." << endl;
	exit(0);
      }
      else m = atoi(argv[f + 1]);
      }
    if(strncmp(argv[f], "-f", 2) == 0) {

      /* Checking freq for T */
      strcpy(freq, "yes");
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  tfreq = atof(argv[f + 1]);

      /* Checking freq for C */
      
      if((argv[f + 2] == NULL) || (atof(argv[f + 2]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  cfreq = atof(argv[f + 2]);

      /* Checking freq for A */
      
      if((argv[f + 3] == NULL) || (atof(argv[f + 3]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  afreq = atof(argv[f + 3]);

      /* Checking freq for G */
      
      if((argv[f + 4] == NULL) || (atof(argv[f + 4]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  gfreq = atof(argv[f + 4]);
    }

    if(strncmp(argv[f], "-t", 2) == 0) {
      strcpy(TsTv, "yes");
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) < 0)){
	cerr << "Must enter an integer for transition bias." << endl;
	exit(0);
      }
      else  trans = atof(argv[f + 1]);
    }

    if(strncmp(argv[f], "-r", 2) == 0)    strcpy(mfile, "yes");
    if(strncmp(argv[f], "-n", 2) == 0)   
      { strcpy(Newick, "yes");
      if(argv[f + 1] == NULL){
	cerr << "Error: Please enter a tree newick file." << endl;
	exit(0);
      } 
      else strcpy(filename, argv[f + 1]);
      if(argv[f + 2] == NULL){
	cerr << "Error: Please enter an alignment file." << endl;
	exit(0);
      }
      else strcpy(AlignFile, argv[f + 2]);
      }
    if(strncmp(argv[f], "-I", 2) == 0)
      { strcpy(iter, "yes"); 
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) != atoi(argv[f+1]))){
	cerr << "Error: Please make sure that m is an integer." << endl;
	exit(0);
      }
      else BignIter = atoi(argv[f + 1]);
      }

    if(strncmp(argv[f], "-i", 2) == 0)
      { strcpy(iter, "yes"); 
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) != atoi(argv[f+1]))){
	cerr << "Error: Please make sure that m is an integer." << endl;
	exit(0);
      }
      else nIter = atoi(argv[f + 1]);
      }
    if(strncmp(argv[f], "-N", 2) == 0)
      {
      if((argv[f + 1] == NULL) ||  (atoi(argv[f + 1]) < 2) || (atof(argv[f + 1]) != atoi(argv[f+1]))){
	cerr << "Error: Please make sure that m is an integer > 1." << endl;
	exit(0);
      }
      else nMWt = atoi(argv[f + 1]);
      }
  }

  if(Newick[0] == 'n')
    strcpy(filename, argv[argc-1]);
  cout << invocation << endl;
 
  if(argc < 2){
    cerr << "Please enter an input file name!" << endl;
    exit(0);
  }
  
  
  /* Setting the GTR model..... */
  if(freq[0] == 'n')
    {FreqVec(0) = 0.25; FreqVec(1) = 0.25;  FreqVec(2) = 0.25;  FreqVec(3) = 0.25;}
  else{ 
    FreqVec(0) = afreq; FreqVec(1) = gfreq;  FreqVec(2) = cfreq;  FreqVec(3) = tfreq; }

  if(freq[0] == 'y') 
    cout << fixed << "Frequencies: T:" << FreqVec(3) << " C:" << FreqVec(2) << " A:" << FreqVec(0) << " G:" << FreqVec(1) << endl;

  if(TsTv[0] == 'y')
    cout << "Transition/transversion ratio is: " << trans << endl;

  Tht(0) = 1;  Tht(1) = 1; Tht(2) = 1;  Tht(3) = 1;  Tht(4) = 1; Tht(5) = 1;
  totalTht(0) = 0;  totalTht(1) = 0; totalTht(2) = 0;  
  totalTht(3) = 0;  totalTht(4) = 0; Tht(5) = 0;
  strcpy(TsTv, "yes");

  /* Initializing variables...... */  
  
  numLvs = 0;
  ct.reset();

  /* Writing tree matrix from DNA sequences using MJOIN to get initial Branchlengths..... */

  PhylMat = writeTreeMatrix(filename, Newick, numLvs, m, BrLenV2, &InputData,
			    FreqVec(3), FreqVec(2),FreqVec(0), FreqVec(1),  
			    freq, TsTv, trans, &listOfNames,  
			    AlignFile);
  TreeNode * initialTree;
  /* Big iteration loop... */
  int BignumNds = numLvs - 1, **SeqMat; 
  int dim = 0;  //  double ** GMat, **FMat;
  long double logL = 0.0;

//   for(int i = 0; i < (2 * numLvs - 1); i++){
//     for(int j = 0; j < 4; j++)
//       cout << PhylMat[i][j] << " ";
//     cout << endl; 
//   }

  SeqMat = DNAMat(InputData, numLvs, SeqLen);
  getEmpFreq(FreqVec, SeqMat,  SeqLen, numLvs, numSt);

  cout << "Estimated Freq: " << endl;
  for(int i = 0; i < numSt; i++) 
    cout << FreqVec(i) << " ";
  cout << endl;
  for(int jj = 0; jj < (2 * numLvs - 2); jj++)
    if(BrLenV2(jj) < 0.000001) BrLenV2(jj) = 0.01;
//   for(int jj = 0; jj < (2 * numLvs - 2); jj++)
//     cout << BrLenV2(jj) << " ";
//   cout << endl;

  Tht(0) = 1;  Tht(1) = 1; Tht(2) = 1;  Tht(3) = 1;  Tht(4) = 1; Tht(5) = 1;
  totalTht(0) = 1;  totalTht(1) = 1; totalTht(2) = 1;  
  totalTht(3) = 1;  totalTht(4) = 1; Tht(5) = 1;
  
  logL = GTR2(numSt, numLvs, numLvs - 1, InputData, SeqLen, PhylMat, FreqVec, 
       Tht, nIter, BrLenV2, nZell);
  
  TreeNode *finalTree = new TreeNode;
  MatrixToTree(numLvs, PhylMat, finalTree, 2*numLvs - 2, BrLenV2, listOfNames);
  cout << "The final tree is: " << endl;
  printDND3(finalTree);
  ofstream finalTreeFile("final.dnd");
  printDND2(finalTree, finalTreeFile);
  cout << endl << "Final Tht: ";
  for(int k = 0; k < Choose(numSt, 2); k++)
    cout << fixed << Tht(k) << " ";
  cout << endl << endl;
  cout << "logL = " << logL << endl;
  ofstream outLog("logL"); 
//   deleteTree(finalTree);
  outLog.precision(10);
  outLog << logL << endl;

  delete [] InputData;

  return 0;
}
