/*********************************************************************
 Author:  Ruriko Yoshida
 Date:    July 2nd, 2004
 Update:  Dec 9th, 2005
 Program: This is software for tree reconstruction from m weights via
          the generalized Neighbor Joining method.

 Input:   The alignment formatted as the multi FASTA format form.

 Output:  A binary tree T which is reconstructed from the input via
          the Neighbor Joining method with our tree matric S.
 
 Modified for EMGNJ.

***********************************************************************/

#include <fstream.h>
#include <iostream.h>
#include <cstdlib>
#include <cstring>
#include <math.h>
#include <algorithm>
#include <time.h>

#include "Taxa.h"
#include "NJTree.h"
#include "Timer.h"
#include "Reading.h"
#include "Index.h"
#include "Cherry.h"
#include "TreeTable.h"
#include "Newick.h"
#include "Seq.h"
#include "Felsenstein.h"

using namespace std;

/* ----------------------------------------------------------------- */
TreeNode* MJOIN(int argc, char *argv[]) {
  char invocation[200], filename[127], mOption[127], freq[127], TsTv[127], mfile[127];
  Timer ct;
  int m = 0, n = 0;
  double tfreq = 0.0, cfreq = 0.0, afreq = 0.0, gfreq = 0.0, trans = 0.0;
  strcpy(mOption, "no");
  strcpy(freq, "no");
  strcpy(TsTv, "no");
  strcpy(mfile, "no");
  cout << "Mjoin v0.9" << endl;
  m = 2;
  strcpy(filename, argv[argc-1]);

  strcpy(invocation, "invocation: ");
  for(int f = 0; f < argc; f++){
    strcat(invocation, argv[f]);
    strcat(invocation, " ");
    if(strncmp(argv[f], "-m", 2) == 0) 
      { strcpy(mOption, "yes"); 
      if((argv[f + 1] == NULL) || (atoi(argv[f + 1]) < 2) || (atof(argv[f + 1]) != atoi(argv[f+1]))){
	cerr << "Error: Please make sure that m is an integer > 1." << endl;
	exit(0);
      }
      else m = atoi(argv[f + 1]);
      }
    if(strncmp(argv[f], "-f", 2) == 0) {

      /* Checking freq for T */
      strcpy(freq, "yes");
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  tfreq = atof(argv[f + 1]);

      /* Checking freq for C */
      
      if((argv[f + 2] == NULL) || (atof(argv[f + 2]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  cfreq = atof(argv[f + 2]);

      /* Checking freq for A */
      
      if((argv[f + 3] == NULL) || (atof(argv[f + 3]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  afreq = atof(argv[f + 3]);

      /* Checking freq for G */
      
      if((argv[f + 4] == NULL) || (atof(argv[f + 4]) < 0)){
	cerr << "Must enter an integer for frequency." << endl;
	exit(0);
      }
      else  gfreq = atof(argv[f + 4]);
    }

    if(strncmp(argv[f], "-t", 2) == 0) {
      strcpy(TsTv, "yes");
      if((argv[f + 1] == NULL) || (atof(argv[f + 1]) < 0)){
	cerr << "Must enter an integer for transition bias." << endl;
	exit(0);
      }
      else  trans = atof(argv[f + 1]);
    }

    if(strncmp(argv[f], "-r", 2) == 0)    strcpy(mfile, "yes");
  }

  if(freq[0] == 'y') 
    cout << "Frequencies: T:" << tfreq << " C:" << cfreq << " A:" << afreq << " G:" << gfreq << endl;

  if(TsTv[0] == 'y')
    cout << "Transition/transversion ratio is: " << trans << endl;
 
  ct.reset();

  /* Reading the input file... */

  if(mfile[0] == 'n'){
    checkMfaFile(filename);
    ReadFMAFileNJ3(n, m, filename, freq, TsTv, tfreq, cfreq, afreq, gfreq, trans);
  }

  /* Reading m-weights.tax file...  */

  ifstream in2;
  if(mfile[0] == 'n')
    in2.open("m-weights.tax");
  else in2.open(filename);
  if(!in2){
    cerr << "There is no such a file." << endl;
      exit(0);
  }
  int numOfLeaves;
  long long int numOfWeights, N = Choose(n, 2);
  Taxa *Data;

  in2 >> numOfLeaves >> numOfWeights; 
  if(numOfWeights != Choose(numOfLeaves, m)) {
    cerr << "Check the input file!  The number of weights is wrong!" << numOfWeights << " " << Choose(numOfLeaves, m) << endl; 
    exit(0);
  }

  Table S(numOfLeaves);

  if(numOfLeaves < m){
    cerr << "The number of taxa must be bigger than m." << endl << endl;
    exit(0);
  }
  else if(numOfLeaves < (2 * m - 1)){
    cout << "\n\nWarning! The number of leaves is less that 2m - 1! \nTree topology and weights might be off!" << endl;
  }

  N = Choose(numOfLeaves, 2);
  Data = new Taxa[numOfWeights];
  for(int i = 0; i < numOfWeights; i++) Data[i].Assgn(m);

  //  S = new Taxa[N](2);

  char* names[numOfLeaves];
  
  ReadTaxaNJ2(Data, numOfLeaves, m, numOfWeights, in2, names);  
  // CreateTreeMatric(Data, numOfLeaves, m, S);
  //testNJ(numOfLeaves, S, names);
  CreateTreeMatric_Table(Data, numOfLeaves, m, S);
  //CreateTree2_Table(S, numOfLeaves, 2, m, N, names);

  return CreateTree(S, numOfLeaves, 2, m, N, names);
}

/* ----------------------------------------------------------------- */
TreeNode* MJOIN(int & n, int m, char *filename, char* freq, char* TsTv, 
		double &tfreq, double &cfreq, double &afreq, double &gfreq, 
		double trans, Alinements ** InputData){
 
  //  Timer ct;
  //Alinements * InputData;
  // tfreq = 0.0, cfreq = 0.0, afreq = 0.0, gfreq = 0.0, trans = 0.0;

  /* Reading the input file... */

    checkMfaFile(filename);

    ReadFMAFileNJ4(n, m, filename, freq, TsTv, tfreq, cfreq, afreq, gfreq, 
		   trans, InputData);

    //    cout << tfreq << " " << cfreq << " " <<  afreq  << " " << gfreq << endl;
    removeDash(InputData, n);

  /* Reading m-weights.tax file...  */

  ifstream in2;
  in2.open("m-weights.tax");
  if(!in2){
    cerr << "There is no such a file." << endl;
      exit(0);
  }

  int numOfLeaves;
  long long int numOfWeights, N = Choose(n, 2);
  Taxa *Data;

  in2 >> numOfLeaves >> numOfWeights; 
  if(numOfWeights != Choose(numOfLeaves, m)) {
    cerr << "Check the input file!  The number of weights is wrong!" << numOfWeights << " " << Choose(numOfLeaves, m) << endl; 
    exit(0);
  }

  Table S(numOfLeaves);

  if(numOfLeaves < m){
    cerr << "The number of taxa must be bigger than m." << endl << endl;
    exit(0);
  }
  else if(numOfLeaves < (2 * m - 1)){
    cout << "\n\nWarning! The number of leaves is less that 2m - 1! \nTree topology and weights might be off!" << endl;
  }

  N = Choose(numOfLeaves, 2);
  Data = new Taxa[numOfWeights];
  for(int i = 0; i < numOfWeights; i++) Data[i].Assgn(m);

  //  S = new Taxa[N](2);

  char* names[numOfLeaves];
  
  ReadTaxaNJ2(Data, numOfLeaves, m, numOfWeights, in2, names);  
  // CreateTreeMatric(Data, numOfLeaves, m, S);
  //testNJ(numOfLeaves, S, names);
  CreateTreeMatric_Table(Data, numOfLeaves, m, S);
  //CreateTree2_Table(S, numOfLeaves, 2, m, N, names);

  return CreateTree(S, numOfLeaves, 2, m, N, names);
}

/* ----------------------------------------------------------------- */
TreeNode* MJOIN(int n, int m, long long int numOfWeights, 
		char** names, long double * totalBrLen){ 
  int numOfLeaves = n;

  long long N = numOfWeights;
  Taxa *Data;

  Table S(numOfLeaves);

  if(numOfLeaves < m){
    cerr << "The number of taxa must be bigger than m." << endl << endl;
    exit(0);
  }
  else if(numOfLeaves < (2 * m - 1)){
    cout << "\n\nWarning! The number of leaves is less that 2m - 1! \nTree topology and weights might be off!" << endl;
  }


  Data = new Taxa[numOfWeights];
  for(int i = 0; i < numOfWeights; i++) Data[i].Assgn(m);

  
  for(long long int f = 0; f < numOfWeights; f++){
    delete [] Data[f].array;
    Data[f].wt = totalBrLen[f];
    delete [] Data[f].edgeWts;
    //    for(int i = 0; i < m; i++) in >> Data[f].edgeWts[i];
  }

  CreateTreeMatric_Table(Data, numOfLeaves, m, S);

  return CreateTree(S, numOfLeaves, 2, m, N, names);
}
/* ----------------------------------------------------------------- */
TreeNode* MJOIN(int & n, int m, char *filename, char* freq, char* TsTv, 
		double &tfreq, double &cfreq, double &afreq, double &gfreq, 
		double trans, Alinements ** InputData, int ***SeqMat, 
		int & SeqLen, int numSt){
  double *Freq;
  Freq = new double[numSt];
  
  for(int i = 0; i < numSt; i++) Freq[i] = 0.0;
  //  Timer ct;
  //Alinements * InputData;
  // tfreq = 0.0, cfreq = 0.0, afreq = 0.0, gfreq = 0.0, trans = 0.0;

  /* Reading the input file... */

    checkMfaFile(filename);

    //  ReadFMAFileNJ4(n, m, filename, freq, TsTv, tfreq, cfreq, afreq, gfreq, 
    //		   trans, InputData);

    ReadFMAFileNJforEMGNJ(n, m, filename, InputData);

    //    cout << tfreq << " " << cfreq << " " <<  afreq  << " " << gfreq << endl;
    removeDash(InputData, n);
    strcpy(freq, "yes");
    (*SeqMat) = DNAMat((*InputData), n, SeqLen);
    getEmpFreq(&Freq, (*SeqMat),  SeqLen, n, numSt);
    afreq = Freq[0]; gfreq = Freq[1]; cfreq = Freq[2]; tfreq = Freq[3];
    cout << fixed << afreq << " " << gfreq << " " <<  cfreq  << " " << tfreq << endl;
    delete [] Freq;
    computeIniTreeforEMGNJ(n, m, freq, TsTv, tfreq, cfreq, afreq, gfreq, 
			   trans, InputData);

  /* Reading m-weights.tax file...  */

  ifstream in2;
  in2.open("m-weights.tax");
  if(!in2){
    cerr << "There is no such a file." << endl;
      exit(0);
  }

  int numOfLeaves;
  long long int numOfWeights, N = Choose(n, 2);
  Taxa *Data;

  in2 >> numOfLeaves >> numOfWeights; 
  if(numOfWeights != Choose(numOfLeaves, m)) {
    cerr << "Check the input file!  The number of weights is wrong!" << numOfWeights << " " << Choose(numOfLeaves, m) << endl; 
    exit(0);
  }

  Table S(numOfLeaves);

  if(numOfLeaves < m){
    cerr << "The number of taxa must be bigger than m." << endl << endl;
    exit(0);
  }
  else if(numOfLeaves < (2 * m - 1)){
    cout << "\n\nWarning! The number of leaves is less that 2m - 1! \nTree topology and weights might be off!" << endl;
  }

  N = Choose(numOfLeaves, 2);
  Data = new Taxa[numOfWeights];
  for(int i = 0; i < numOfWeights; i++) Data[i].Assgn(m);

  //  S = new Taxa[N](2);

  char* names[numOfLeaves];
  
  ReadTaxaNJ2(Data, numOfLeaves, m, numOfWeights, in2, names);  
  // CreateTreeMatric(Data, numOfLeaves, m, S);
  //testNJ(numOfLeaves, S, names);
  CreateTreeMatric_Table(Data, numOfLeaves, m, S);
  //CreateTree2_Table(S, numOfLeaves, 2, m, N, names);

  return CreateTree(S, numOfLeaves, 2, m, N, names);
}
